import CodeGen from '@alilc/lowcode-code-generator';
import vue3 from './plugins/project/vue3';
import vue from './plugins/component/vue3/vue';
import methods from './plugins/component/vue3/methods';
import mock from './plugins/component/vue3/mock';
import lifeCycles from './plugins/component/vue3/lifeCycles';
import styles from './plugins/component/vue3/styles';
import initState from './plugins/component/vue3/initState';
import * as constants from './plugins/component/vue3/const';
import * as globalUtils from './utils';
import { prettier } from './postprocessor';

/**
 * 出码插件入口
 *
 * 用于出码插件初始化以及完成各子插槽函数的注册
 *
 * 插槽函数：用于解析schema配置生成特定代码片段。
 *
 * 插槽：用于指定代码片段最终插入的目标位置。
 */
export default function createVueProjectBuilder() {
  return CodeGen.createProjectBuilder({
    // 注册项目模板插槽
    template: vue3.template,
    // 处理对应插槽的函数注册，key:[a(),b(),c()]；key为对应插槽，a,b,c为插槽对应的处理函数
    plugins: {
      // 组件插槽函数
      components: [],
      // 页面插槽函数
      pages: [
      // 解析数据源配置，生成数据mock，模拟fetch
      mock(),
      // 初始化绑定数据
      initState(),
      // 解析并生成vue生命周期函数
      lifeCycles(),
      // 解析并生成函数
      methods(),
      // 解析生成页面样式
      styles(),
      // vue文件生成，主要用于将schema文件中的相关配置转化为实际的vue代码
      vue({
        nodeTypeMapping: {
          Div: 'div',
          Component: 'div',
          Page: 'div',
          Block: 'div'
        }
      })],
      // 路由插槽函数，用于生成路由
      router: [vue3.plugins.router()],
      // 入口插槽函数注册，用于生成main.js
      entry: [vue3.plugins.entry()],
      // 主文件入口插槽函数，用于生成app.vue
      app: [vue3.plugins.app()],
      // 主样式文件入口插槽函数，用于生成全局样式文件
      mainCss: [vue3.plugins.mainCss()],
      // html插槽函数，用于生成html模板文件
      htmlEntry: [vue3.plugins.entryHtml()],
      // pkg插槽函数，用于生成项目工程的package.json
      packageJSON: [vue3.plugins.packageJSON()]
    },
    // 后置处理器，用于出码解析完成后的处理，例如：生成代码的格式化
    postProcessors: [prettier()]
  });
}
var publishers = CodeGen.publishers;
export { publishers };
export var plugins = {
  methods: methods,
  vue: vue,
  styles: styles,
  constants: constants,
  globalUtils: globalUtils
};