import _extends from "@babel/runtime/helpers/extends";
import _ from 'lodash';
import { pipe } from 'fp-ts/function';
import { isNodeSchema } from '@alilc/lowcode-types';
import CodeGen, { CodeGeneratorError, PIECE_TYPE } from '@alilc/lowcode-code-generator';
import { executeFunctionStack } from './aopHelper';
import { encodeJsxStringNode } from './encodeJsxAttrString';
import { unwrapJsExprQuoteInJsx } from './jsxHelpers';
import { transformThis2Context } from '../plugins/component/vue3/core/jsx/handlers/transformThis2Context';
var _CodeGen$utils = CodeGen.utils,
  generateCompositeType = _CodeGen$utils.compositeType.generateCompositeType,
  getStaticExprValue = _CodeGen$utils.common.getStaticExprValue;
function mergeNodeGeneratorConfig(cfg1, cfg2) {
  if (cfg2 === void 0) {
    cfg2 = {};
  }
  var resCfg = {};
  if (cfg1.handlers || cfg2.handlers) {
    resCfg.handlers = _extends({}, cfg1.handlers || {}, cfg2.handlers || {});
  }
  if (cfg1.tagMapping || cfg2.tagMapping) {
    resCfg.tagMapping = cfg2.tagMapping || cfg1.tagMapping;
  }
  if (cfg1.attrPlugins || cfg2.attrPlugins) {
    var _resCfg$attrPlugins, _resCfg$attrPlugins2;
    resCfg.attrPlugins = [];
    (_resCfg$attrPlugins = resCfg.attrPlugins).push.apply(_resCfg$attrPlugins, cfg2.attrPlugins || []);
    (_resCfg$attrPlugins2 = resCfg.attrPlugins).push.apply(_resCfg$attrPlugins2, cfg1.attrPlugins || []);
  }
  if (cfg1.nodePlugins || cfg2.nodePlugins) {
    var _resCfg$nodePlugins, _resCfg$nodePlugins2;
    resCfg.nodePlugins = [];
    (_resCfg$nodePlugins = resCfg.nodePlugins).push.apply(_resCfg$nodePlugins, cfg2.nodePlugins || []);
    (_resCfg$nodePlugins2 = resCfg.nodePlugins).push.apply(_resCfg$nodePlugins2, cfg1.nodePlugins || []);
  }
  return resCfg;
}
export function isPureString(v) {
  // FIXME: 目前的方式不够严谨
  return v[0] === "'" && v[v.length - 1] === "'" || v[0] === '"' && v[v.length - 1] === '"';
}
function generateJSExpressionAttrName(name) {
  var newName = ":" + name;
  var vuePa = ['v-model', 'v-bind'];
  vuePa.forEach(function (item) {
    if (name.startsWith(item)) {
      newName = name;
    }
  });
  return newName;
}
function generateAttrValue(attrData, scope, config) {
  var _ref = attrData,
    attrValue = _ref.attrValue;
  if ((attrValue === null || attrValue === void 0 ? void 0 : attrValue.type) === 'JSExpression') {
    attrData.attrValue = "" + attrValue.value;
    attrData.attrName = generateJSExpressionAttrName(attrData.attrName);
  }
  var valueStr = generateCompositeType(attrData.attrValue, scope, {
    handlers: config === null || config === void 0 ? void 0 : config.handlers,
    nodeGenerator: config === null || config === void 0 ? void 0 : config.self
  });
  return [{
    type: PIECE_TYPE.ATTR,
    name: attrData.attrName,
    value: valueStr
  }];
}
function generateAttr(attrName, attrValue, scope, config) {
  var pieces;
  if (config !== null && config !== void 0 && config.attrPlugins) {
    pieces = executeFunctionStack({
      attrName: attrName,
      attrValue: attrValue
    }, scope, config.attrPlugins, generateAttrValue, config);
  } else {
    pieces = generateAttrValue({
      attrName: attrName,
      attrValue: attrValue
    }, scope, config);
  }
  pieces = pieces.map(function (p) {
    // FIXME: 在经过 generateCompositeType 处理过之后，其实已经无法通过传入值的类型判断传出值是否为纯字面值字符串了（可能包裹了加工函数之类的）
    //        因此这个处理最好的方式是对传出值做语法分析，判断以哪种模版产出 Attr 值
    var newValue;
    if (p.value && isPureString(p.value)) {
      // 似乎多次一举，目前的诉求是处理多种引号类型字符串的 case，正确处理转义
      var content = getStaticExprValue(p.value);
      newValue = JSON.stringify(encodeJsxStringNode(content));
    } else {
      p.name = ":" + p.name;
      newValue = "'" + p.value + "'";
    }
    // vue 的插槽特殊写法单独处理：v-slot="slotname" => #slotname
    if (p.name === 'v-slot') {
      return {
        value: "#" + p.value.replaceAll('"', ''),
        type: PIECE_TYPE.ATTR
      };
    }
    return {
      value: p.name + "=" + newValue,
      type: PIECE_TYPE.ATTR
    };
  });
  return pieces;
}
function generateAttrs(nodeItem, scope, config) {
  var props = nodeItem.props;
  var pieces = [];
  if (props) {
    if (!Array.isArray(props)) {
      Object.keys(props).forEach(function (propName) {
        if (propName) {
          pieces = pieces.concat(generateAttr(propName, props[propName], scope, config));
        }
      });
    } else {
      props.forEach(function (prop) {
        if (prop.name && !prop.spread) {
          pieces = pieces.concat(generateAttr(prop.name, prop.value, scope, config));
        }

        // TODO: 处理 spread 场景（<Xxx {...(something)}/>)
        // 这种在 schema 里面怎么描述
      });
    }
  }

  return pieces;
}
function generateBasicNode(nodeItem, scope, config) {
  var pieces = [];
  var tagName = ((config === null || config === void 0 ? void 0 : config.tagMapping) || _.identity)(nodeItem.componentName);
  pieces.push({
    value: tagName || '',
    // FIXME: type detection error
    type: PIECE_TYPE.TAG
  });
  return pieces;
}

// 处理vue特有的插槽
function slotHelper(nodeItem) {
  var props = nodeItem.props,
    children = nodeItem.children;
  _.forEach(props, function (obj, key) {
    if ((obj === null || obj === void 0 ? void 0 : obj.type) === 'JSSlot') {
      var _obj$value = obj.value,
        value = _obj$value === void 0 ? [] : _obj$value;
      var oldChild = children !== null && children !== void 0 ? children : [];
      if (typeof children === 'string') {
        oldChild = [children];
      }
      nodeItem.children = [].concat(oldChild, [{
        componentName: 'template',
        props: {
          'v-slot': key
        },
        children: value
      }]);
      delete props[key];
    }
  });
  return nodeItem;
}
function generateSimpleNode(nodeItem, scope, config) {
  var newNodeItem = slotHelper(nodeItem);
  var basicParts = generateBasicNode(newNodeItem, scope, config) || [];
  var attrParts = generateAttrs(newNodeItem, scope, config) || [];
  var childrenParts = [];
  if (nodeItem.children && config !== null && config !== void 0 && config.self) {
    var childrenStr = config.self(newNodeItem.children, scope);
    childrenParts.push({
      type: PIECE_TYPE.CHILDREN,
      value: childrenStr
    });
  }
  return [].concat(basicParts, attrParts, childrenParts);
}
function linkPieces(pieces) {
  var tagsPieces = pieces.filter(function (p) {
    return p.type === PIECE_TYPE.TAG;
  });
  if (tagsPieces.length !== 1) {
    throw new CodeGeneratorError('Only one tag definition required', tagsPieces);
  }
  var tagName = tagsPieces[0].value;
  var beforeParts = pieces.filter(function (p) {
    return p.type === PIECE_TYPE.BEFORE;
  }).map(function (p) {
    return p.value;
  }).join('');
  var afterParts = pieces.filter(function (p) {
    return p.type === PIECE_TYPE.AFTER;
  }).map(function (p) {
    return p.value;
  }).join('');
  var childrenParts = pieces.filter(function (p) {
    return p.type === PIECE_TYPE.CHILDREN;
  }).map(function (p) {
    return p.value;
  }).join('');
  var attrsParts = pieces.filter(function (p) {
    return p.type === PIECE_TYPE.ATTR;
  }).map(function (p) {
    return p.value;
  }).join(' ');
  attrsParts = attrsParts ? " " + attrsParts : '';
  if (childrenParts) {
    return beforeParts + "<" + tagName + attrsParts + ">" + childrenParts + "</" + tagName + ">" + afterParts;
  }
  return beforeParts + "<" + tagName + attrsParts + " />" + afterParts;
}
function generateNodeSchema(nodeItem, scope, config) {
  var pieces = [];
  if (config !== null && config !== void 0 && config.nodePlugins) {
    var res = executeFunctionStack(nodeItem, scope, config.nodePlugins, generateSimpleNode, config);
    pieces.push.apply(pieces, res);
  } else {
    pieces.push.apply(pieces, generateSimpleNode(nodeItem, scope, config));
  }
  return linkPieces(pieces);
}

// TODO: 生成文档
// 为包裹的代码片段生成子上下文，集成父级上下文，并传入子级上下文新增内容。（如果存在多级上下文怎么处理？）
// 创建新的上下文，并从作用域中取对应同名变量塞到作用域里面？
// export function createSubContext() {}

/**
 * JSX 生成逻辑插件。在 React 代码模式下生成 loop 相关的逻辑代码
 * @type NodePlugin Extended
 *
 * @export
 * @param {IPublicTypeNodeSchema} nodeItem 当前 UI 节点
 * @returns {CodePiece[]} 实现功能的相关代码片段
 */
export function generateReactLoopCtrl(nodeItem, scope, config, next) {
  if (nodeItem.loop) {
    var _nodeItem$loopArgs, _nodeItem$loopArgs2;
    var loopItemName = ((_nodeItem$loopArgs = nodeItem.loopArgs) === null || _nodeItem$loopArgs === void 0 ? void 0 : _nodeItem$loopArgs[0]) || 'item';
    var loopIndexName = ((_nodeItem$loopArgs2 = nodeItem.loopArgs) === null || _nodeItem$loopArgs2 === void 0 ? void 0 : _nodeItem$loopArgs2[1]) || 'index';

    // 新建作用域
    var subScope = scope.createSubScope([loopItemName, loopIndexName]);
    var pieces = next ? next(nodeItem, subScope, config) : [];

    // 生成循环变量表达式
    var loopDataExpr = pipe(nodeItem.loop,
    // 将 JSExpression 转换为 JS 表达式代码:
    function (expr) {
      return generateCompositeType(expr, scope, {
        handlers: config === null || config === void 0 ? void 0 : config.handlers,
        tolerateEvalErrors: false // 这个内部不需要包 try catch, 下面会统一加的
      });
    },
    // 将 this.xxx 转换为 __$$context.xxx:
    function (expr) {
      return transformThis2Context(expr, scope, {
        ignoreRootScope: true
      });
    }
    // 如果要容忍错误，则包一层 try catch (基于助手函数 __$$evalArray)
    // (expr) => (tolerateEvalErrors ? `__$$evalArray(() => (${expr}))` : expr),
    );

    pieces.unshift({
      value: "(" + loopDataExpr + ").map((" + loopItemName + ", " + loopIndexName + ") => ((__$$context) => (",
      type: PIECE_TYPE.BEFORE
    });
    pieces.push({
      value: "))(__$$createChildContext(__$$context, { " + loopItemName + ", " + loopIndexName + " })))",
      type: PIECE_TYPE.AFTER
    });
    return pieces;
  }
  return next ? next(nodeItem, scope, config) : [];
}

/**
 * JSX 生成逻辑插件。在 React 代码模式下生成 condition 相关的逻辑代码
 * @type NodePlugin
 *
 * @export
 * @param {IPublicTypeNodeSchema} nodeItem 当前 UI 节点
 * @returns {CodePiece[]} 实现功能的相关代码片段
 */
export function generateConditionReactCtrl(nodeItem, scope, config, next) {
  var pieces = next ? next(nodeItem, scope, config) : [];
  if (nodeItem.condition != null && nodeItem.condition !== true) {
    var value = generateCompositeType(nodeItem.condition, scope, {
      handlers: config === null || config === void 0 ? void 0 : config.handlers
    });
    pieces.unshift({
      value: "!!(" + value + ") && (",
      type: PIECE_TYPE.BEFORE
    });
    pieces.push({
      value: ')',
      type: PIECE_TYPE.AFTER
    });
  }
  return pieces;
}

/**
 * JSX 生成逻辑插件。在 React 代码模式下，如果 Node 生成结果是一个表达式，则对其进行 { Expression } 包装
 * @type NodePlugin
 *
 * @export
 * @param {IPublicTypeNodeSchema} nodeItem 当前 UI 节点
 * @returns {CodePiece[]} 实现功能的相关代码片段
 */
export function generateReactExprInJS(nodeItem, scope, config, next) {
  var pieces = next ? next(nodeItem, scope, config) : [];
  if (nodeItem.condition != null && nodeItem.condition !== true || nodeItem.loop != null) {
    pieces.unshift({
      value: '{',
      type: PIECE_TYPE.BEFORE
    });
    pieces.push({
      value: '}',
      type: PIECE_TYPE.AFTER
    });
  }
  return pieces;
}
var handleChildren = function handleChildren(v) {
  return v.join('');
};
var handleEvent = function handleEvent(props) {
  if (props === void 0) {
    props = {};
  }
  var result = _extends({}, props);
  var redundancePro = ['onClick', '__events'];
  redundancePro.forEach(function (property) {
    if (result.hasOwnProperty(property)) {
      delete result[property];
    }
  });
  return result;
};
export function createNodeGenerator(cfg) {
  if (cfg === void 0) {
    cfg = {};
  }
  var generateNode = function generateNode(nodeItem, scope) {
    if (_.isArray(nodeItem)) {
      var resList = nodeItem.map(function (n) {
        return generateNode(n, scope);
      });
      return handleChildren(resList);
    }
    if (isNodeSchema(nodeItem)) {
      var props = nodeItem.props,
        loop = nodeItem.loop;
      if (loop) {
        var items = JSON.stringify(loop);
        nodeItem.props = _extends({}, props, {
          'v-for': "(item,index) in " + items
        });
      }
      if (props !== null && props !== void 0 && props.__events) {
        var newProps = handleEvent(props);
        nodeItem.props = newProps;
      }
      return generateNodeSchema(nodeItem, scope, _extends({}, cfg, {
        self: generateNode
      }));
    }
    var valueStr = generateCompositeType(nodeItem, scope, {
      handlers: cfg.handlers,
      nodeGenerator: generateNode
    });
    if (isPureString(valueStr)) {
      return encodeJsxStringNode(getStaticExprValue(valueStr));
    }
    return "{" + valueStr + "}";
  };
  return function (nodeItem, scope) {
    return unwrapJsExprQuoteInJsx(generateNode(nodeItem, scope));
  };
}
export function generateVue(nodeItem, scope, config, next) {
  var pieces = next ? next(nodeItem, scope, config) : [];
  if (nodeItem.condition != null && nodeItem.condition !== true || nodeItem.loop != null) {
    pieces.unshift({
      value: '{',
      type: PIECE_TYPE.BEFORE
    });
    pieces.push({
      value: '}',
      type: PIECE_TYPE.AFTER
    });
  }
  return pieces;
}
var defaultReactGeneratorConfig = {
  // nodePlugins: [generateReactExprInJS, generateReactLoopCtrl, generateConditionReactCtrl],
  nodePlugins: []
};
export function createReactNodeGenerator(cfg) {
  var newCfg = mergeNodeGeneratorConfig(defaultReactGeneratorConfig, cfg);
  return createNodeGenerator(newCfg);
}