"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugins = exports.publishers = void 0;
const tslib_1 = require("tslib");
const lowcode_code_generator_1 = tslib_1.__importDefault(require("@alilc/lowcode-code-generator"));
const vue3_1 = tslib_1.__importDefault(require("./plugins/project/vue3"));
const vue_1 = tslib_1.__importDefault(require("./plugins/component/vue3/vue"));
const methods_1 = tslib_1.__importDefault(require("./plugins/component/vue3/methods"));
const mock_1 = tslib_1.__importDefault(require("./plugins/component/vue3/mock"));
const lifeCycles_1 = tslib_1.__importDefault(require("./plugins/component/vue3/lifeCycles"));
const styles_1 = tslib_1.__importDefault(require("./plugins/component/vue3/styles"));
const initState_1 = tslib_1.__importDefault(require("./plugins/component/vue3/initState"));
const constants = tslib_1.__importStar(require("./plugins/component/vue3/const"));
const globalUtils = tslib_1.__importStar(require("./utils"));
const postprocessor_1 = require("./postprocessor");
/**
 * 出码插件入口
 *
 * 用于出码插件初始化以及完成各子插槽函数的注册
 *
 * 插槽函数：用于解析schema配置生成特定代码片段。
 *
 * 插槽：用于指定代码片段最终插入的目标位置。
 */
function createVueProjectBuilder() {
    return lowcode_code_generator_1.default.createProjectBuilder({
        // 注册项目模板插槽
        template: vue3_1.default.template,
        // 处理对应插槽的函数注册，key:[a(),b(),c()]；key为对应插槽，a,b,c为插槽对应的处理函数
        plugins: {
            // 组件插槽函数
            components: [],
            // 页面插槽函数
            pages: [
                // 解析数据源配置，生成数据mock，模拟fetch
                (0, mock_1.default)(),
                // 初始化绑定数据
                (0, initState_1.default)(),
                // 解析并生成vue生命周期函数
                (0, lifeCycles_1.default)(),
                // 解析并生成函数
                (0, methods_1.default)(),
                // 解析生成页面样式
                (0, styles_1.default)(),
                // vue文件生成，主要用于将schema文件中的相关配置转化为实际的vue代码
                (0, vue_1.default)({
                    nodeTypeMapping: {
                        Div: 'div',
                        Component: 'div',
                        Page: 'div',
                        Block: 'div',
                    },
                }),
            ],
            // 路由插槽函数，用于生成路由
            router: [vue3_1.default.plugins.router()],
            // 入口插槽函数注册，用于生成main.js
            entry: [vue3_1.default.plugins.entry()],
            // 主文件入口插槽函数，用于生成app.vue
            app: [vue3_1.default.plugins.app()],
            // 主样式文件入口插槽函数，用于生成全局样式文件
            mainCss: [vue3_1.default.plugins.mainCss()],
            // html插槽函数，用于生成html模板文件
            htmlEntry: [vue3_1.default.plugins.entryHtml()],
            // pkg插槽函数，用于生成项目工程的package.json
            packageJSON: [vue3_1.default.plugins.packageJSON()],
        },
        // 后置处理器，用于出码解析完成后的处理，例如：生成代码的格式化
        postProcessors: [(0, postprocessor_1.prettier)()],
    });
}
exports.default = createVueProjectBuilder;
exports.publishers = lowcode_code_generator_1.default.publishers;
exports.plugins = {
    methods: methods_1.default,
    vue: vue_1.default,
    styles: styles_1.default,
    constants,
    globalUtils,
};
//# sourceMappingURL=index.js.map