"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports["default"] = createVueProjectBuilder;
exports.publishers = exports.plugins = void 0;
var _lowcodeCodeGenerator = _interopRequireDefault(require("@alilc/lowcode-code-generator"));
var _vue = _interopRequireDefault(require("./plugins/project/vue3"));
var _vue2 = _interopRequireDefault(require("./plugins/component/vue3/vue"));
var _methods = _interopRequireDefault(require("./plugins/component/vue3/methods"));
var _mock = _interopRequireDefault(require("./plugins/component/vue3/mock"));
var _lifeCycles = _interopRequireDefault(require("./plugins/component/vue3/lifeCycles"));
var _styles = _interopRequireDefault(require("./plugins/component/vue3/styles"));
var _initState = _interopRequireDefault(require("./plugins/component/vue3/initState"));
var constants = _interopRequireWildcard(require("./plugins/component/vue3/const"));
var globalUtils = _interopRequireWildcard(require("./utils"));
var _postprocessor = require("./postprocessor");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * 出码插件入口
 *
 * 用于出码插件初始化以及完成各子插槽函数的注册
 *
 * 插槽函数：用于解析schema配置生成特定代码片段。
 *
 * 插槽：用于指定代码片段最终插入的目标位置。
 */
function createVueProjectBuilder() {
  return _lowcodeCodeGenerator["default"].createProjectBuilder({
    // 注册项目模板插槽
    template: _vue["default"].template,
    // 处理对应插槽的函数注册，key:[a(),b(),c()]；key为对应插槽，a,b,c为插槽对应的处理函数
    plugins: {
      // 组件插槽函数
      components: [],
      // 页面插槽函数
      pages: [
      // 解析数据源配置，生成数据mock，模拟fetch
      (0, _mock["default"])(),
      // 初始化绑定数据
      (0, _initState["default"])(),
      // 解析并生成vue生命周期函数
      (0, _lifeCycles["default"])(),
      // 解析并生成函数
      (0, _methods["default"])(),
      // 解析生成页面样式
      (0, _styles["default"])(),
      // vue文件生成，主要用于将schema文件中的相关配置转化为实际的vue代码
      (0, _vue2["default"])({
        nodeTypeMapping: {
          Div: 'div',
          Component: 'div',
          Page: 'div',
          Block: 'div'
        }
      })],
      // 路由插槽函数，用于生成路由
      router: [_vue["default"].plugins.router()],
      // 入口插槽函数注册，用于生成main.js
      entry: [_vue["default"].plugins.entry()],
      // 主文件入口插槽函数，用于生成app.vue
      app: [_vue["default"].plugins.app()],
      // 主样式文件入口插槽函数，用于生成全局样式文件
      mainCss: [_vue["default"].plugins.mainCss()],
      // html插槽函数，用于生成html模板文件
      htmlEntry: [_vue["default"].plugins.entryHtml()],
      // pkg插槽函数，用于生成项目工程的package.json
      packageJSON: [_vue["default"].plugins.packageJSON()]
    },
    // 后置处理器，用于出码解析完成后的处理，例如：生成代码的格式化
    postProcessors: [(0, _postprocessor.prettier)()]
  });
}
var publishers = _lowcodeCodeGenerator["default"].publishers;
exports.publishers = publishers;
var plugins = {
  methods: _methods["default"],
  vue: _vue2["default"],
  styles: _styles["default"],
  constants: constants,
  globalUtils: globalUtils
};
exports.plugins = plugins;